% R91922034 Hung-Te Lin, AI Project 2

% Utility Functions

% extract_regions: extract region data from assigned map
%                  usage: extract_regions(EdgeList, [], Regions).
%  unify the bindings and regions
extract_regions([], Regions, Regions).
%  recursive rules
extract_regions([ [S,T] | Rest ], Bindings, Regions) :-
	member(S, Bindings) ->
	(member(T, Bindings) ->
	    % both S and T in Regions
	    extract_regions( Rest, Bindings, Regions ) ;
	    % S in Regions while T is not
	    extract_regions( Rest, [T|Bindings], Regions)) ;
	% S not in Regions
	(member(T, Bindings) ->
	    % T in Regions while S in not
	    extract_regions( Rest, [S|Bindings], Regions) ;
	    % both S and T are not in Regions
	    extract_regions(Rest, [S,T|Bindings], Regions)).
	

% paint_colors: paint colors to regions
%               usage: paint_colors(Regions, Colors, Paintings).
%  first rule: if no region, just return nothing
paint_colors([], _, []).
%  search rule:
paint_colors([Rgn | Rest], Colors, [[Rgn,C] | Paintings]) :-
	member(C, Colors),
	paint_colors(Rest, Colors, Paintings).
%  utility to display all paintings.
%list-paintings(Regions, Colors) :-
%	paint_colors(Regions, Colors, Paintings),
%	write(Paintings),nl,
%	fail.

% same_color: check if any adjacent edges have same_color
%             usage: same_color(EdgeList, Colors, Paintings).
same_color(EdgeList, Colors, Paintings) :-
	member(C, Colors),
	member([S, T], EdgeList),
	member([S, C], Paintings),
	member([T, C], Paintings).

% the final program, in term project #2 specified format.
color(EdgeList, Colors, Coloring) :-
	extract_regions(EdgeList, [], Regions_BeforeSorting),
	sort(Regions_BeforeSorting, Regions),
	paint_colors(Regions, Colors, Coloring),
	\+ same_color(EdgeList, Colors, Coloring).

list-color(EdgeList, Colors, Coloring) :-
	extract_regions(EdgeList, [], Regions_BeforeSorting),
	sort(Regions_BeforeSorting, Regions),
	paint_colors(Regions, Colors, Coloring),
	\+ same_color(EdgeList, Colors, Coloring),
	write(Coloring), nl,
	fail.
